import os
import sys
import pathlib
from functools import partial
from PySide2 import QtCore, QtGui, QtWidgets
from sohstationviewer.view.ui.main_ui import Ui_MainWindow
from sohstationviewer.view.calendardialog import CalendarDialog
from sohstationviewer.view.ui.calendar_widget import CalendarWidget

class MainWindow(QtWidgets.QMainWindow, Ui_MainWindow):
    """
    Implements the core logic for the Ui_MainWindow class
    produced by Qt Designer. Any custom slots / signals
    should be implemented in this class. Any modifications to
    Ui_MainWindow *will* be lost if the UI is ever updated in
    Qt Designer.
    """

    def __init__(self, parent=None):
        super().__init__(parent)
        self.setupUi(self)

        cwd = os.path.dirname(pathlib.Path().absolute())
        self.currentDirectoryChanged.emit(cwd)

        # File menu
        self.deleteSetup.triggered.connect(self.deleteSetupFile)

        # Commands menu
        self.openGPSPlots.triggered.connect(self.openGpsPlot)
        self.searchLog.triggered.connect(self.openLogSearch)
        self.plotTimeRanges.triggered.connect(self.openPlotTimeRanges)
        self.plotPositions.triggered.connect(self.openPlotPositions)

        # Commands/Export submenu
        self.exportDeploymentFile.triggered.connect(self.exportDepLine)
        self.exportDeploymentBlock.triggered.connect(self.exportDepBlock)
        self.exportTSPGeometry.triggered.connect(self.exportTspGeom)
        self.exportShotInfo.triggered.connect(self.exportShotInf)

        # Help menu
        self.openCalendar.triggered.connect(self.openCalendarWidget)

        # Options Menu
        self.sortGroup = QtWidgets.QActionGroup(self)
        self.sortGroup.addAction(self.sortFilesByType)
        self.sortGroup.addAction(self.sortFilesAlphabetically)
        self.sortFilesByType.setChecked(True)

        self.colorGroup = QtWidgets.QActionGroup(self)
        self.colorGroup.addAction(self.colorMPRegular)
        self.colorGroup.addAction(self.colorMPTrillium)
        self.colorMPRegular.setChecked(True)

        self.dateGroup = QtWidgets.QActionGroup(self)
        self.dateGroup.addAction(self.showYYYYDOYDates)
        self.dateGroup.addAction(self.showYYYY_MM_DDDates)
        self.dateGroup.addAction(self.showYYYYMMMDDDates)
        self.showYYYYDOYDates.setChecked(True)

        self.timeFromPushButton.clicked.connect(self.onStartTimeCalendar)
        self.timeToPushButton.clicked.connect(self.onEndTimeCalendar)

        self.timeToDateEdit.setCalendarWidget(CalendarWidget(self))
        self.timeToDateEdit.setDate(QtCore.QDate.currentDate())

        self.timeFromDateEdit.setCalendarWidget(CalendarWidget(self))
        self.timeFromDateEdit.setDate(QtCore.QDate.currentDate())

    @QtCore.Slot()
    def deleteSetupFile(self):
        print('Deleting setup file.')

    @QtCore.Slot()
    def openGpsPlot(self):
        print('Opening GPS plot.')

    @QtCore.Slot()
    def openLogSearch(self):
        print('Opening Log search.')

    @QtCore.Slot()
    def openPlotTimeRanges(self):
        print('Opening Time Ranges Plot.')

    @QtCore.Slot()
    def openPlotPositions(self):
        print('Opening Positions Plot.')

    @QtCore.Slot()
    def exportDepLine(self):
        print('Exporting Deployment File (Line).')

    @QtCore.Slot()
    def exportDepBlock(self):
        print('Exporting Deployment File (Block).')

    @QtCore.Slot()
    def exportTspGeom(self):
        print('Exporting TSP Shot File / Geometry.')

    @QtCore.Slot()
    def exportShotInf(self):
        print('Exporting Shot Info.')

    @QtCore.Slot()
    def readSelectedFile(self):
        print('Reading currently selected file.')

    @QtCore.Slot()
    def stopFileRead(self):
        print('Abandoning file read.')

    @QtCore.Slot()
    def writePSFile(self):
        print('Writing PS file.')

    @QtCore.Slot()
    def reloadFile(self):
        print('Reloading last file.')

    @QtCore.Slot(str)
    def changeCurrentDirectory(self, path=''):
        fd = QtWidgets.QFileDialog(self)
        fd.setFileMode(QtWidgets.QFileDialog.Directory)
        fd.exec_()
        new_path = fd.selectedFiles()[0]
        self.currentDirectoryChanged.emit(new_path)

    currentDirectoryChanged = QtCore.Signal(str)

    @QtCore.Slot()
    def openCalendarWidget(self):
        calendar = CalendarDialog(self)
        calendar.show()

    @QtCore.Slot()
    def setStartTime(self, calendar):
        # TODO: date needs to be formatted
        # according to the options
        textField = self.timeFromLineEdit
        date = calendar.calendarWidget.selectedDate()
        textField.setText(date.toString())

    @QtCore.Slot()
    def setEndTime(self, calendar):
        # TODO: date needs to be formatted
        # according to the options
        textField = self.timeToLineEdit
        date = calendar.calendarWidget.selectedDate()
        textField.setText(date.toString())


    @QtCore.Slot()
    def onStartTimeCalendar(self):
        # Want to re-use the calendar dialog rather than having special ones
        # for each case where it is used. Also need to detect when the dialog
        # is closed, so we can get the date that the user selected. So we need
        # to override the closeEvent method. A cleaner way to do this
        # would  be to invoke an optional callback function from closeEvent
        # defined within the calendar dialog itself, if the callback has been set.
        # However, I didn't even think of that until after I
        # implemented this, because I haven't had coffee yet.
        # I think the best way to do this may actually be to set up a slot to listen
        # for the selectionChanged signal...
        calendar = CalendarDialog(self)
        calendar.accept = partial(MainWindow.setStartTime, self, calendar)
        calendar.show()

    @QtCore.Slot()
    def onEndTimeCalendar(self):
        calendar = CalendarDialog(self)
        calendar.accept = partial(MainWindow.setEndTime, self, calendar)
        calendar.show()


if __name__ == '__main__':
    app = QtWidgets.QApplication(sys.argv)
    wnd = MainWindow()
    wnd.show()
    sys.exit(app.exec_())

